package learningRules;

import java.util.HashSet;
import java.util.Iterator;

import utilities.Individual;


/**
* This class is a Condorcet Learning Rule.  An individual
* looks around and adopts the strategey that the
* majority in his neighborhood are adopting.
* @Author Kevin J.S. Zollman
*/

public class CondorcetLearning implements LearningRule {

	// The number of types of beliefs
	int strats;
	
	/**
	 * There is no updating so this function does nothing
	 */
	public void newGeneration(Individual i) {}
	
	/**
	 * This function censuses individual in the neighborhood
	 * and adopts the strategy used by the majority last
	 * round.
	 */
	public int pickStrat(Individual i) {
		int[] count = new int[strats];
		int total = 0;
		
		for(int k=0;k<strats;k++)
			count[k] = 0;
		
		HashSet neigh = i.getConnected();
        synchronized(neigh) {
        Iterator j = neigh.iterator();
        while(j.hasNext()) {
        	Individual n = (Individual) j.next();
        	
        	count[n.getLastStrat()]++;
        	total++;
        }
        }
    	count[i.getLastStrat()]++;
    	total++;
    	
    	for(int k=0;k<strats;k++) {
    		if(count[k] > (total-count[k]))
    			return k;
    		
    	}
    	return i.getLastStrat();
	}
	
    /**
     * All this does is update LastStrat.
     */
    public void processPayoff(Individual i) {
    	i.setLastStrat();
    }

    /**
     * A constructor which sets the number of strats
     */
    public CondorcetLearning(int s) {strats = s;}
    
    /**
     * A constructor which automatically sets the number
     * of strats at 2
     */
    public CondorcetLearning() {strats = 2;}

}