package learningRules;
import java.util.*;

import utilities.Game;
import utilities.Individual;
import utilities.ModelError;


/**
* This is a learning rule where an individual adopts the strategy that did best
* on the previous round from among those strategies choosen by a neighbor.  If 
* another strategy is tied with the players current strategy, the player retains
* her current strategy.  If two or more strategies are tied, but both better than her 
* current strategy, she chooses amongst those at random.
* @author Kevin Zollman
*/

public class ImitateBestLearning implements LearningRule {

        Game game;
        
        /**
        * This does nothing since there is no discounting or anything.
        * @param i The individual calling the function
        * @throws ModelError (although, since this does nothing it will never be thrown)
        */
        public void newGeneration(Individual i) throws ModelError { }
        
        /**
        * Processing a payoff does nothing, since there are no beliefs to be updated.
        * @param i The individual calling the function
        * @throws ModelError (although, since this does nothing it will never be thrown)
        */
        public void processPayoff(Individual i) throws ModelError { }
        
        /**
        * This function chooses the strategy which did best on the previous round.
        * If the players previous strategy is tied with another
        * as the best, the player keeps the strategy.  If two strategies (not used by the
        * player on the last round) are tied, the player chooses one at random.
        * @param i The calling individual
        */
        public int pickStrat(Individual i) {
                
                int defstrat = i.getLastStrat();
                int defstratpayoff = i.getLastPayoff();
                
                int bestpayoff = defstratpayoff;
                int[] beststrats = new int[game.getStrats()];
                int numbeststrats = 1;
                beststrats[0] = defstrat;
                
                //gets a list of neighbors actions
                                
                HashSet neigh = i.getConnected();
                synchronized(neigh) {
                Iterator j = neigh.iterator();
                while(j.hasNext()) {
                        Individual n = (Individual) j.next();
                        boolean test=true;
                        
                        for(int k=0;k<numbeststrats;k++) {
                                if(beststrats[k] == n.getLastStrat()) {
                                        test=false;
                                        break;
                                }
                        }
                        if(test) {
                                if(n.getLastPayoff()>defstratpayoff &&
                                        n.getLastPayoff()>=bestpayoff) {
                                        if(n.getLastPayoff()==bestpayoff) {
                                                beststrats[numbeststrats] = n.getLastStrat();
                                                numbeststrats++;
                                        }
                                        else {
                                                bestpayoff=n.getLastPayoff();
                                                beststrats[0] = n.getLastStrat();
                                                numbeststrats = 1;
                                        }
                                }
                        }
                }
                }
                
                return beststrats[i.getRandom().nextInt(numbeststrats)];
        }
        
        /**
        * Constructor
        * @param g The game
        */
        public ImitateBestLearning(Game g) { game = g; }
}
