package learningRules;
import utilities.Individual;
import utilities.ModelError;

/**
* Simple reinforcement learning for strategies
*/
public class StratReinforce implements LearningRule {
        
        double scale;
        
        /**
        * Prepares the individual for a new generation.  Simply aplies the discount factor
        * to weights
        * @param i The calling individual
        */
        public synchronized void newGeneration(Individual i) throws ModelError {
                for(int j=0;j<i.getWeights().length;j++) 
                        i.setStratWeights(j, (double) i.getWeights(j)*i.getDiscount());
        }
        
        /**
        * Processes the payoff by adding the payoff to the weight of the strategy choosen on this
        * round
        * @param i The calling individual
        */
        public synchronized void processPayoff(Individual i) throws ModelError {
                i.addStratWeight(i.getStrat(), (double) scale*i.getPayoff());
        }
        
        /**
        * Chooses a strategy based on standard reinforcement method
        * @param i The calling individual
        */
        public synchronized int pickStrat(Individual i) {
                double total = 0.0;
                double randnum;
                
                for(int j=0;j<i.getWeights().length;j++)
                        total += i.getWeights(j);
                randnum = i.getRandom().nextDouble();
                double subtotal = i.getWeights(0);
                for(int j=0;j<i.getWeights().length;j++) {
                        if(randnum <= ((double) subtotal / total)) 
                                return j;
                        subtotal += i.getWeights(j+1);
                }
                
                return -1;
        }

        /**
        * This takes a multiplicative scalling factor so that the weights don't grow extremely
        * large (outside the realm of double)
        * @param s The scaling factor
        */
        
        public StratReinforce(double s) { scale = s; }
}
