package models;
import utilities.Game;
import utilities.Individual;
import utilities.MersenneTwister;
import utilities.ModelError;
import learningRules.LearningRule;
import learningRules.PayoffLearning;
import learningRules.SmoothPayoffLearning;
import learningRules.StratReinforce;

import com.martiansoftware.jsap.*;

import connectionRules.ConnectNothing;
import connectionRules.ConnectionRule;

public class SingleDPL {
        
        public static final MersenneTwister random = new MersenneTwister();
        
        Individual individual;	
        /**
	 * Returns the value of indivis.
	 */
	public Individual getIndivids() { return individual; }

	/**
	 * Sets the value of indivis.
	 * @param indivis The value to assign indivis.
	 */
        public void setIndividual(Individual individs) {this.individual = individs;}
        
        
        int optimal;
        
        Game game;	
        /**
	 * Returns the value of game.
	 */
	public Game getGame(){ 	return game;	}
        
	/**
	 * Sets the value of game.
	 * @param game The value to assign game.
	 */
	public void setGame(Game game)	{this.game = game;}
        
        /**
        * Resets then plays a generation.
        * This first calls each individuals reset function, then calls each individuals playGame
        * function, then calles each individuals update function
        * @throws ModelError If something bad happens
        */
        public synchronized void playGen() throws ModelError {
                individual.reset();
                individual.playGame();
                individual.update();
        }
                
        public void run(JSAPResult config) throws ModelError {
                if(config.getInt("verbose")==0) {
                         System.out.println("WARNING: Don't use this verbosity level here.");
                }
                        
                double avepayoff;
                long[] actioncount = new long[config.getInt("stratnum")];
                for(long i=0L;i<config.getLong("runs");i++) {
                        individual.randomAB(config.getInt("alphabeta"), config.getInt("alphabeta"), config.getInt("stratnum"));
                        
                        
                        avepayoff = 0.0;
                        for(int k=0;k<config.getInt("stratnum");k++) 
                                actioncount[k] = 0L;
                        int j=0;
                        for(j=0;j<config.getInt("generations");j++) { 
                                
                                if(config.getInt("verbose") ==2) {
                                        String beliefstring = "";
                                        if(config.getString("learningrule").equals("r")) {
                                                double total = 0.0;
                                                for(int k=0;k<config.getInt("stratnum");k++) 
                                                        total += individual.getWeights(k);
                                                for(int k=0;k<config.getInt("stratnum");k++) {
                                                        beliefstring += (double) individual.getWeights(k)/total;
                                                        beliefstring += " ";
                                                }
                                        }
                                        else {
                                                for(int k=0;k<config.getInt("stratnum");k++) {
                                                        beliefstring += individual.getAlpha(k);
                                                        beliefstring += " ";
                                                        beliefstring += individual.getBeta(k);
                                                        beliefstring += " ";
                                                }
                                        }
                                        System.out.print(i + " " + j + " " + beliefstring);
                                        if(j==0) {
                                                System.out.println("- 0");
                                        }
                                        else {
                                                System.out.println(individual.getStrat() + " " + individual.getPayoff());
                                        }
                                }                                
                                        
                                playGen();
                                        
                                // update statistics
                                
                                avepayoff += (double) individual.getPayoff()/config.getInt("generations");
                                actioncount[individual.getStrat()]++;
                               
                        }
                        switch(config.getInt("verbose")) {
                        case 0:
                                if(i%250==0) 
                                        System.out.print("*");
                                break;
                        case 1: 
                              String beliefstring = "";
                              if(config.getString("learningrule").equals("re")) {
                                        double total = 0.0;
                                        for(int k=0;k<config.getInt("stratnum");k++) 
                                                total += individual.getWeights()[k];
                                        for(int k=0;k<config.getInt("stratnum");k++) {
                                                beliefstring += (double) individual.getWeights()[k]/total;
                                                beliefstring += " ";
                                        }
                                }
                                else {
                                        for(int k=0;k<config.getInt("stratnum");k++) {
                                                beliefstring += individual.getAlpha(k);
                                                beliefstring += " ";
                                                beliefstring += individual.getBeta(k);
                                                beliefstring += " ";
                                        }
                                }
                                String actioncountstr = "";
                                for(int k=0;k<config.getInt("stratnum");k++) { 
                                        actioncountstr += actioncount[k];
                                        actioncountstr += " ";
                                }
                                System.out.println(i + " "  + avepayoff + " " + actioncountstr + beliefstring + j);
                                break;
                        case 2:
                                System.out.println("** New run **");
                                break;
                        }
                }
                
        }
        
        
        public SingleDPL(JSAPResult config) throws ModelError{
                int[][] payoffs;
                optimal=0;
                if(config.getInt("stratnum") != config.getIntArray("payoffs").length) {
                        payoffs = new int[config.getInt("stratnum")][config.getInt("stratnum")];
                        for(int i=0;i<config.getInt("stratnum");i++) {
                                if(i<config.getIntArray("payoffs").length) {
                                        for(int j=0;j<config.getInt("stratnum");j++)
                                                payoffs[i][j] = config.getIntArray("payoffs")[i];
                                        if(payoffs[i][0] > config.getInt("bigpayoff")) {
                                                System.err.println();
                                                System.err.println("Payoff cannot be larger than big payoff");
                                                System.exit(1);
                                        }
                                }
                                else {
                                        int number = PayoffLearningModel.random.nextInt(config.getInt("bigpayoff"));
                                        for(int j=0;j<config.getInt("stratnum");j++) 
                                                payoffs[i][j] = number;
                                }
                                if((payoffs[i][0] == payoffs[optimal][0]) && (i!=0)) {
                                        System.err.println();
                                        System.err.println("Cannot have two payoffs that are identical");
                                        System.exit(1);
                                }
                                if(payoffs[i][0] > payoffs[optimal][0])
                                        optimal = i;
                                
                        }
                }
                else {
                        payoffs = new int[config.getInt("stratnum")][config.getInt("stratnum")];
                        for(int i=0;i<config.getInt("stratnum");i++) {
                                for(int j=0;j<config.getInt("stratnum");j++) 
                                        payoffs[i][j] = config.getIntArray("payoffs")[i];
                                
                                if((payoffs[i][0] == payoffs[optimal][0]) && (i!=0)) {
                                        System.err.println();
                                        System.err.println("Cannot have two payoffs that are identical: " 
                                                + i + " " + optimal);
                                        System.exit(1);
                                }
                                if(payoffs[i][0] > payoffs[optimal][0])
                                        optimal = i;
                                
                        }
                }
                
                //creates a new game with payoffs, it is stochastic, the rng, and its not strategic
                Game game = new Game(true, SingleDPL.random);
                game.setMatrix(payoffs, config.getInt("bigpayoff"));
                game.setStrategic(false);
                
                ConnectionRule c = new ConnectNothing();
                
                 // Set the learning rule
                LearningRule lr;
                if(config.getString("learningrule").equals("fp")) {
                        lr = new PayoffLearning(game);
                }
                else {
                        if(config.getString("learningrule").equals("sfp")) {
                                lr = new SmoothPayoffLearning(game, config.getDouble("gamma"));
                        }
                        else {
                                lr = new StratReinforce(0.01);
                        }
                }
                
                double[] zc = new double[game.getStrats()];
                for(int j=0;j<game.getStrats();j++)
                        zc[j] = 0.0;
                individual = new Individual(1);
                Individual[] network = new Individual[1];
                network[0] = individual;
                individual.setNetwork(network);
                individual.setRandom(SingleDPL.random);
                individual.setConnectionRule(c);
                individual.setLearningRule(lr);
                individual.setGame(game);
                individual.setMutation(config.getDouble("mutationRate"));
                individual.setConstraint(zc);
        }
}
