/**
 * 
 */
package mutationRules;

import utilities.Individual;
import utilities.ModelError;

/**
 * Mutates an agents beliefs, this includes strategy
 * weights, beliefs, and alpha and beta levels.
 * @author Kevin J.S. Zollman
 *
 */
public class MutateBeliefs implements MutationRule {

	double mutationrate;
	
	/**
	 * Mutates weights, beliefs, and alpha/beta values.
	 * 
	 */
	public void mutate(Individual i) throws ModelError {
		if(mutationrate > 0.0) {
            if(i.getRandom().nextDouble() <= mutationrate) {
                    i.clearStratWeights();
                    i.randBeliefs(i.getGame().getStrats(), false);
                    i.randomAB();
            }
		}

	}

	/**
	 * Sets the mutation rate
	 */
	public void setMutationRate(double m, int s) throws ModelError {
		if((m<0.0)||(m>1.0))
            throw new ModelError("Mutation not [0,1]");
		mutationrate = m;
	}
	
	public double getMutationRate() { return mutationrate; }
	
	/**
	 * Constructor which accepts rate and strategy number.
	 */
	public MutateBeliefs(double m) throws ModelError {
		if((m<0.0)||(m>1.0))
            throw new ModelError("Mutation not [0,1]");
		mutationrate = m;
	}
	
	/**
	 * Constructor which sets mutation rate to 0.
	 */
	public MutateBeliefs() {
		mutationrate = 0.0;
	}
	
}
